# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pansy.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='pansy.proto',
  package='pansy.proto',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0bpansy.proto\x12\x0bpansy.proto\"Y\n\nRspToUsers\x12)\n\x04rows\x18\x01 \x03(\x0b\x32\x1b.pansy.proto.RspToUsers.Row\x1a \n\x03Row\x12\x0c\n\x04uids\x18\x01 \x03(\x03\x12\x0b\n\x03\x62uf\x18\x02 \x01(\x0c\"\x1a\n\nCloseUsers\x12\x0c\n\x04uids\x18\x01 \x03(\x03\x62\x06proto3')
)




_RSPTOUSERS_ROW = _descriptor.Descriptor(
  name='Row',
  full_name='pansy.proto.RspToUsers.Row',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uids', full_name='pansy.proto.RspToUsers.Row.uids', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='buf', full_name='pansy.proto.RspToUsers.Row.buf', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=117,
)

_RSPTOUSERS = _descriptor.Descriptor(
  name='RspToUsers',
  full_name='pansy.proto.RspToUsers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rows', full_name='pansy.proto.RspToUsers.rows', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RSPTOUSERS_ROW, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=117,
)


_CLOSEUSERS = _descriptor.Descriptor(
  name='CloseUsers',
  full_name='pansy.proto.CloseUsers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uids', full_name='pansy.proto.CloseUsers.uids', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=145,
)

_RSPTOUSERS_ROW.containing_type = _RSPTOUSERS
_RSPTOUSERS.fields_by_name['rows'].message_type = _RSPTOUSERS_ROW
DESCRIPTOR.message_types_by_name['RspToUsers'] = _RSPTOUSERS
DESCRIPTOR.message_types_by_name['CloseUsers'] = _CLOSEUSERS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RspToUsers = _reflection.GeneratedProtocolMessageType('RspToUsers', (_message.Message,), {

  'Row' : _reflection.GeneratedProtocolMessageType('Row', (_message.Message,), {
    'DESCRIPTOR' : _RSPTOUSERS_ROW,
    '__module__' : 'pansy_pb2'
    # @@protoc_insertion_point(class_scope:pansy.proto.RspToUsers.Row)
    })
  ,
  'DESCRIPTOR' : _RSPTOUSERS,
  '__module__' : 'pansy_pb2'
  # @@protoc_insertion_point(class_scope:pansy.proto.RspToUsers)
  })
_sym_db.RegisterMessage(RspToUsers)
_sym_db.RegisterMessage(RspToUsers.Row)

CloseUsers = _reflection.GeneratedProtocolMessageType('CloseUsers', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEUSERS,
  '__module__' : 'pansy_pb2'
  # @@protoc_insertion_point(class_scope:pansy.proto.CloseUsers)
  })
_sym_db.RegisterMessage(CloseUsers)


# @@protoc_insertion_point(module_scope)
